import pandas as pd

from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.pdfpage import PDFPage
from pdfminer.converter import TextConverter
from pdfminer.layout import LAParams

from io import StringIO

def from_csv(file_path, head_column='None'):
    if file_path.endswith('.csv'):
        df = pd.read_csv(file_path)

        if head_column != 'None':
            try:
                df_dict = df.set_index(head_column).T.to_dict()
            except KeyError:
                print(head_column, 'not in columns.', '\nColumns Found:', list(df.columns))
                df_dict = {}
        else:
            id_columns = [c for c in list(df.columns) if 'id ' in c.lower() + ' ']
            if len(id_columns) == 1:
                selected_column = id_columns[0]
                df_dict = df.set_index(selected_column).T.to_dict()
            else:
                print('Multiple ids columns found... Define one id column in "head_column" argument')
                df_dict = {}
    else:
        print("Input file is not csv ...")
        df_dict = {}

    return df_dict

def from_tsv(file_path, head_column='None'):
    
    if file_path.endswith('.tsv'):
        df = pd.read_csv(file_path, sep='\t')

        if head_column != 'None':
            try:
                df_dict = df.set_index(head_column).T.to_dict()
            except KeyError:
                print(head_column, 'not in columns.', '\nColumns Found:', list(df.columns))
                df_dict = {}
        else:
            id_columns = [c for c in list(df.columns) if 'id ' in c.lower() + ' ']
            if len(id_columns) == 1:
                selected_column = id_columns[0]
                df_dict = df.set_index(selected_column).T.to_dict()
            else:
                print('Multiple ids columns found... Define one id column in "head_column" argument')
                df_dict = {}
    else:
        print("Input file is not tsv ...")
        df_dict = {}

    return df_dict


def from_xlsx(file_path, head_column='None'):
    
    if file_path.endswith('.xlsx'):
        df = pd.read_excel(file_path)
        if head_column != 'None':
            try:
                df_dict = df.set_index(head_column).T.to_dict()
            except KeyError:
                print(head_column, 'not in columns.', '\nColumns Found:', list(df.columns))
                df_dict = {}
        else:
            id_columns = [c for c in list(df.columns) if 'id ' in c.lower() + ' ']
            if len(id_columns) == 1:
                selected_column = id_columns[0]
                df_dict = df.set_index(selected_column).T.to_dict()
            else:
                print('Multiple ids columns found... Define one id column in "head_column" argument')
                df_dict = {}
    else:
        print("Input file is not xlsx ...")
        df_dict = {}

    return df_dict

def from_pdf(file_path, starting_page='None', ending_page='None'):
    
    if file_path.endswith('.pdf'):
        print('Converting pdf to text ...')
        rsrcmgr = PDFResourceManager()
        sio = StringIO()
        codec = 'utf-8'
        laparams = LAParams()
        device = TextConverter(rsrcmgr, sio, codec=codec, laparams=laparams)
        interpreter = PDFPageInterpreter(rsrcmgr, device)

        fp = open(file_path, 'rb')
        if starting_page != 'None' and ending_page != 'None':
            for page in list(PDFPage.get_pages(fp))[starting_page-1:ending_page]:
                interpreter.process_page(page)
        elif starting_page != 'None':
            for page in list(PDFPage.get_pages(fp))[starting_page-1:]:
                interpreter.process_page(page)
        elif ending_page != 'None':
            for page in list(PDFPage.get_pages(fp))[:ending_page]:
                interpreter.process_page(page)
        else:
            for page in list(PDFPage.get_pages(fp)):
                interpreter.process_page(page)
        fp.close()

        text = sio.getvalue()

        device.close()
        sio.close()
        data = {'text': text}
    else:
        print("Input file is not pdf ...")
        data = {}

    return data


def from_txt(file_path):
    
    if file_path.endswith('.txt'):
        with open(file_path, 'r') as file:
            text = file.read().replace('\n', '')
            data = {'text': text}
    else:
        print("Input file is not txt ...")
        data = {}
    
    return data



import json
import requests


def get_data(password, concept_list, filters='None'):
    concept_list = '|'.join(concept_list)
    if filters != 'None':
        filters = '|'.join(filters)
    
    response = requests.get('https://cloud.bolooba.com:25556/adverse_effects/' + password + '/' + concept_list + '/' + filters)
    if response.status_code == 200:
        data = dict(response.json())
        
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data



from collections import Counter

def suggest_belief(password, query, topn=10):
    import requests, json
    response = requests.post('https://cloud.bolooba.com:25556/suggest_belief/' + password + '/' + str(topn), json=json.dumps(query))
    if response.status_code == 200:
        data = dict(response.json())
        data = sorted(data.items(), key=lambda x: x[1], reverse=True)
        data = dict(data)
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data

def compute_prob(password, query):
    import requests, json
    response = requests.post('https://cloud.bolooba.com:25556/tzager_prob/' + password, json=json.dumps(query))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data

def targeted_prob(password, query, a_rate=2, report=False):
    import requests, json
    response = requests.post('https://cloud.bolooba.com:25556/targeted_prob/' + password + '/' + str(a_rate), json=json.dumps(query))
    if response.status_code == 200:
        data = dict(response.json())
        if report:
            print('================= REPORT =================')
            
            
            print('\n\n----------------------------- Probabilities Data --------------------\n\n')
            print('Probability:', data['step1']['prob'], '\n')
            print('Probability analysis per given variable:')
            for g in data['step1']['prob per given variable']:
                print(g, ':', data['step1']['prob per given variable'][g])
    
            print('\n\n--- Probabilities Analysis For Suggested Beliefs (based on given) ---\n\n')
            temp_data = []
            for b in data['step2']:
                temp_data.append((b, data['step2'][b]['prob'], data['step2'][b]['prob per given variable']))
            temp_data = sorted(temp_data, key=lambda x: x[1], reverse=True)
            for pair in temp_data:
                b = pair[0]
                prob = pair[1]
                per_given_v = pair[2]
                print('Belief:', b)
                print('Probability:', prob)
                print('Probability analysis per given variable:')
                for g in per_given_v:
                    print(g, ':', per_given_v[g])
                print()
            print("\n\n-------------- Missing Given variables to complete Belief ------------\n\n")
            for categ in data['step3']:
                print(categ + ' missing:', [c for c in data['step3'][categ] if c not in query['given_variables']])
                print()
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data


import pandas as pd
import json
import requests

def from_xls(password, file_path):

    df = pd.read_excel(file_path)
    data = df.to_dict('dict')
    columns = df.columns
    entities = df['Entities']
    samples_dict = {}
    for c in columns:
        if 'Sample' in c:
            samples_dict[c] = {}
            samples = df[c]
            for i, s in enumerate(samples):
                if s > 0:
                    samples_dict[c][entities[i]] = s
    
    response = requests.post('https://cloud.bolooba.com:25556/predictors/' + password , json=json.dumps(samples_dict))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data


def from_csv(password, file_path):

    df = pd.read_csv(file_path)
    data = df.to_dict('dict')
    columns = df.columns
    entities = df['Entities']
    samples_dict = {}
    for c in columns:
        if 'Sample' in c:
            samples_dict[c] = {}
            samples = df[c]
            for i, s in enumerate(samples):
                if s > 0:
                    samples_dict[c][entities[i]] = s
    
    response = requests.post('https://cloud.bolooba.com:25556/predictors/' + password , json=json.dumps(samples_dict))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data





from tqdm import tqdm
from collections import Counter


def values_for_variables(data):
    all_possible_variables = {}

    for id in data:
        for variable in data[id]:
            value = data[id][variable]
            try:
                all_possible_variables[variable].append(str(value))
            except KeyError:
                all_possible_variables[variable] = []
                all_possible_variables[variable].append(str(value))

    for variable in all_possible_variables:
        all_possible_variables[variable] = list(set(all_possible_variables[variable]))
    return all_possible_variables


def conditional_prob(data, q):
    number_of_samples = len(data)
    variables_join = {}
    query_variable = q['query_variable']['variable'] + '===' + str(q['query_variable']['value'])
    evidence_variable = [x['variable'] + '===' + str(x['value']) for x in q['evidence_variables']]
    for id in data:
        instance = []
        for variable in data[id]:
            value = data[id][variable]
            instance.append(variable + '===' + str(value))
        variables_join[id] = instance

    if not evidence_variable:
        query_hits = 0
        for id in variables_join:
            if query_variable in variables_join[id]:
                query_hits += 1
        total_pr = round(query_hits/number_of_samples, 3)
    else:
        evidence_query_join = 0
        evidence_join = 0
        for id in variables_join:
            if set(evidence_variable) <= set(variables_join[id]):
                evidence_join += 1
            if set(evidence_variable + [query_variable]) <= set(variables_join[id]):
                evidence_query_join += 1
        if evidence_join != 0:
            total_pr = round(evidence_query_join/evidence_join, 3)
        else:
            total_pr = 0
    return total_pr


def prob_event(data, variables, q, topn=5):
    import itertools
    query_variable = q['query_variable']
    evidence_variables = q['evidence_variables']
    data_values = {}
    for ev in evidence_variables:
        data_values[ev] = variables[ev]
    
    cases = [dict(zip(data_values, x)) for x in itertools.product(*data_values.values())]
    scores = []
    with tqdm(total=len(cases)) as pbar:
        for case in cases:
            q_prime = {'query_variable': None, 'evidence_variables': []}
            q_prime['query_variable'] = query_variable
            for v in case:
                q_prime['evidence_variables'].append({'variable':v, 'value': case[v]})
            prob = conditional_prob(data, q_prime)
            pbar.update(1)
            scores.append((q_prime, prob))
    
    scores =  sorted(scores, key=lambda x: x[1], reverse=True)[:topn]
    for s in scores:
        print('For case:', s[0])
        print('Pr:', s[1])
        print()
    return scores

